##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

from Interface import Base
class IForum(Base):
    def posts(REQUEST=None):
        """ returns the top level postings """
    def getForum():
        """ returns the Forum object """
    def generate_id():
        """ returns a available id """
    def addPost(id='', title='', body='', author='', email='', in_reply_to='', REQUEST=None):
        """ adds a Posting to current Forum """
    def addForum(id, title='', REQUEST=None):
        """ constructs a new Forum in the container it is called in """
