## Script (Python) "forum_post_toSearch"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

dest = context.getForum().absolute_url() + '/forum_search_form'
context.REQUEST.RESPONSE.redirect(dest)
