from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import setDefaultRoles

# Gathering Event Related Permissions into one place
ViewForum = 'View Forum'
AddForumPosting = 'Add Forum posting'
AddForum = 'Add Forum'

# Set up default roles for permissions
setDefaultRoles(AddForumPosting,
                ('Manager', 'Member', 'Anonymous'))

setDefaultRoles(ViewForum,
                ('Manager', 'Member', 'Anonymous'))

setDefaultRoles(AddForum,
                ('Manager', 'Member'))
